package main;

import main.*;
import wad.*;
import gui.*;

import java.io.*;
import javax.swing.*;

public class Main
{
	public static String version = "v0.3 Alpha";
	public static String config = "strife";
	public static Config configuration;
	public static Wad iwad;
	public static Wad pwad;
	public static Palette playpal;
	public static Font font;
	public static GUI gui;
	public static DialogBlock page = null;
	public static Script script = null;

	public static void main(String[] args)
	{
		JFrame.setDefaultLookAndFeelDecorated(true);
		System.out.println("Reading Config");
		String argName = null; //read in any params.
		for(int i = 0;i < args.length;i++)
		{
			if(args.length == 1)
			{
				argName = "file";
			}
			if(argName == null)
			{
				if(args[i].equals("-config"))
				{
					argName = "config";
				}
				else if(args[i].equals("-file"))
				{
					argName = "file";
				}
			}
			else
			{
				if(argName.equals("config"))
				{
					config = args[i];
				}
				else if(argName.equals("file"))
				{
					pwad = new Wad(args[i]);
				}
				argName = null;
			}
		}
		File configFile = new File(config + ".cfg");
		if(!configFile.exists())
		{
			fatalError("Config file does not exist.");
			return;
		}
		configuration = new Config(config + ".cfg");
		System.out.println("Reading IWad");
		iwad = new Wad(configuration.iwad);
		if(pwad == null && configuration.pwad.equals("") == false)
		{
			pwad = new Wad(configuration.pwad);
		}
		if(pwad != null)
		{
			System.out.println("Reading PWad");
			short[] configData = pwad.getDataForLump(pwad.getLumpByName("DLEDCONF"));
			configuration.readConfigLump(configData);
		}
		System.out.println("Retrieving Palette");
		Lump paletteLump = iwad.getLumpByName("PLAYPAL");
		if(pwad != null && pwad.getLumpByName("PLAYPAL") != null) //if a palette exists in the pwad load it instead.
		{
			paletteLump = pwad.getLumpByName("PLAYPAL");
		}
		if(paletteLump == null)
		{
			fatalError("Could not find palette in wad(s).");
		}
		playpal = new Palette(iwad.getDataForLump(paletteLump, 768));
		System.out.println("Preparing Font");
		font = new Font(configuration.font);
		System.out.println("Init GUI");
		gui = new GUI();
		if(script == null) //first startup
		{
			script = Script.newScript();
		}
		else //reboot
		{
			gui.setPage(script.NPCs.get(0).pages.get(0));
		}
	}
	public static void fatalError(Exception e, String description)
	{
		System.out.println("Fatal Error: " + description);
		e.printStackTrace();
		System.exit(0);
	}
	public static void fatalError(String description)
	{
		System.out.println("Fatal Error: " + description);
		System.exit(0);
	}
	public static void error(String description)
	{
		System.out.println("Error: " + description);
	}
	public static void error(Exception e, String description)
	{
		System.out.println("Error: " + description);
		e.printStackTrace();
	}
	public static void setPage(DialogBlock page)
	{
		Main.page = page;
		gui.setPage(page);
		updateBackdrop();
	}
	public static void updateBackdrop()
	{
		boolean usePwad = false;
		Lump backdrop = iwad.getLumpByName(page.getPanel());
		if(backdrop == null && pwad != null)
		{
			if(pwad.getLumpByName(page.getPanel()) != null)
			{
				backdrop = pwad.getLumpByName(page.getPanel());
				usePwad = true;
			}
		}
		if(backdrop != null)
		{
			if(usePwad)
			{
				gui.dialog.setBackdrop(pwad.getDataForLump(backdrop));
				gui.dialogCopy.setBackdrop(pwad.getDataForLump(backdrop));
			}
			else
			{
				gui.dialog.setBackdrop(iwad.getDataForLump(backdrop));
				gui.dialogCopy.setBackdrop(iwad.getDataForLump(backdrop));
			}
		}
		else
		{
			gui.dialog.removeBackdrop();
		}
	}
	public static void saveScript(File file)
	{
		try
		{
			FileOutputStream out = new FileOutputStream(file);
			out.write(script.makeScript().trim().getBytes());
			out.close();
		}
		catch(Exception e)
		{
			error(e, "Could not save.");
		}
	}
	//Right now this is designed to read only files generated by this program 
	//it expects everything to be picture perfect (well almost anyways)... So 
	//I will have to rewrite it some day... I think.
	public static Script readScript(File file)
	{
		Script script = new Script();
		NPC npc = null;
		DialogBlock block = null;
		DialogChoice choice = null;
		int activeNPC = -1;
		boolean inPage = false;
		boolean inChoice = false;
		try
		{
			BufferedReader in = new BufferedReader(new FileReader(file));
			String line = null;
			while((line = in.readLine()) != null)
			{
				line = line.replace('	', ' ').trim(); //tabs to spaces
				if(line.lastIndexOf("#") != -1)
				{
					line = line.substring(0, line.lastIndexOf("#")).trim();
				}
				if(line.length() == 0)
				{
					continue;
				}
				String command = null;
				String value = null;
				if(line.indexOf(" ") != -1)
				{
					command = line.substring(0, line.indexOf(" ")).trim();
					value = line.substring(line.indexOf(" ")+1).trim();
				}
				else
				{
					command = line.trim();
					value = "";
				}
				if(!inPage) //NPC Level
				{
					if(command.equals("ACTOR"))
					{
						if(Integer.parseInt(value) != activeNPC)
						{
							npc = new NPC();
							npc.actor = Integer.parseInt(value);
							activeNPC = npc.actor;
							script.NPCs.add(npc);
						}
						block = new DialogBlock();
						inPage = true;
					}
				}
				else //Page/Choice level
				{
					if(!inChoice) //page
					{
						if(command.equals("DROP"))
						{
							block.drop = Integer.parseInt(value);
						}
						else if(command.equals("IF_ITEM"))
						{
							block.if_item[0] = Integer.parseInt(value);
						}
						else if(command.equals("IF_ITEM2"))
						{
							block.if_item[1] = Integer.parseInt(value);
						}
						else if(command.equals("IF_ITEM3"))
						{
							block.if_item[2] = Integer.parseInt(value);
						}
						else if(command.equals("GOTO"))
						{
							block.if_item_goto = Integer.parseInt(value);
						}
						else if(command.equals("NAME"))
						{
							if(value.indexOf("\"") != -1)
							{
								block.setName(value.substring(value.indexOf("\"")+1, value.lastIndexOf("\"")));
							}
							else
							{
								block.setName(value);
							}
						}
						else if(command.equals("VOICE"))
						{
							if(value.indexOf("\"") != -1)
							{
								block.setVoice(value.substring(value.indexOf("\"")+1, value.lastIndexOf("\"")));
							}
							else
							{
								block.setVoice(value);
							}
						}
						else if(command.equals("PANEL"))
						{
							if(value.indexOf("\"") != -1)
							{
								block.setPanel(value.substring(value.indexOf("\"")+1, value.lastIndexOf("\"")));
							}
							else
							{
								block.setPanel(value);
							}
						}
						else if(command.equals("DIALOG"))
						{
							if(value.indexOf("\"") != -1)
							{
								block.setDialog(value.substring(value.indexOf("\"")+1, value.lastIndexOf("\"")));
							}
							else
							{
								block.setDialog(value);
							}
						}
						else if(command.equals("END"))
						{
							npc.pages.add(block);
							block = null;
							inPage = false;
						}
						else if(command.equals("CHOICE"))
						{
							choice = new DialogChoice();
							inChoice = true;
						}
					}
					else //choice
					{
						if(command.equals("GIVE"))
						{
							choice.give = Integer.parseInt(value);
						}
						else if(command.equals("TEXT"))
						{
							if(value.indexOf("\"") != -1)
							{
								choice.setText(value.substring(value.indexOf("\"")+1, value.lastIndexOf("\"")));
							}
							else
							{
								choice.setText(value);
							}
						}
						else if(command.equals("LOG"))
						{
							choice.log = Integer.parseInt(value);
						}
						else if(command.equals("YES"))
						{
							if(value.indexOf("\"") != -1)
							{
								choice.yes = value.substring(value.indexOf("\"")+1, value.lastIndexOf("\""));
							}
							else
							{
								choice.yes = value;
							}
						}
						else if(command.equals("NO"))
						{
							if(value.indexOf("\"") != -1)
							{
								choice.no = value.substring(value.indexOf("\"")+1, value.lastIndexOf("\""));
							}
							else
							{
								choice.no = value;
							}
						}
						else if(command.equals("COST"))
						{
							if(value.indexOf(" * ") == -1)
							{
								continue;
							}
							int item = Integer.parseInt(value.substring(0, value.indexOf(" * ")).trim()) - 1;
							int cost = Integer.parseInt(value.substring(value.indexOf(" * ")+3).trim());
							choice.cost = item;
							choice.cost_check = cost;
							choice.noDisplayCost = false;
						}
						else if(command.equals("NODISPLAYCOST"))
						{
							if(value.indexOf(" * ") == -1)
							{
								continue;
							}
							int item = Integer.parseInt(value.substring(0, value.indexOf(" * ")).trim()) - 1;
							int cost = Integer.parseInt(value.substring(value.indexOf(" * ")+3).trim());
							choice.cost = item;
							choice.cost_check = cost;
							choice.noDisplayCost = true;
						}
						else if(command.equals("ALTCOST"))
						{
							if(value.indexOf(" * ") == -1)
							{
								continue;
							}
							int item = Integer.parseInt(value.substring(0, value.indexOf(" * ")).trim()) - 1;
							int cost = Integer.parseInt(value.substring(value.indexOf(" * ")+3).trim());
							choice.altcost = item;
							choice.altcost_check = cost;
						}
						else if(command.equals("LINK"))
						{
							int link = Integer.parseInt(value);
							if(link < 0)
							{
								link *= -1;
								link--;
								choice.showNext = true;
							}
							else if(link != 0)
							{
								link--;
								choice.showNext = false;
							}
							choice.link = link;
						}
						else if(command.equals(";"))
						{
							block.choices.add(choice);
							choice = null;
							inChoice = false;
						}
					}
				}
			}
			in.close();
		}
		catch(Exception e)
		{
			error(e, "Could not read script.");
		}
		if(script.NPCs.size() == 0 || script.NPCs.get(0).pages.size() == 0)
		{
			return Script.newScript();
		}
		script.NPCs.get(0).updateModels();
		return script;
	}
}